﻿using System;
using System.Windows.Input;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace UniversalSample.Commons
{
    /// <summary>
    /// 네비게이션 관련 헬퍼
    /// </summary>
    public class FrameHelper
    {
        private static FrameHelper _instance;
        /// <summary>
        /// Singleton
        /// </summary>
        public static FrameHelper Instance
        {
            get
            {
                return _instance = _instance ?? new FrameHelper();
            }
        }

        /// <summary>
        /// Frame object
        /// </summary>
        internal Frame _frame;

        /// <summary>
        /// Navigated Command
        /// </summary>
        public ICommand NavigatedCommand { get; set; }
    
        /// <summary>
        /// Navigating Command
        /// </summary>
        public ICommand NavigatingCommand { get; set; }

        /// <summary>
        /// RegisterFrame 
        /// </summary>
        /// <param name="frame"></param>
        public void RegisterFrame(Frame frame)
        {
            _frame = frame;
            if (_frame == null) return;

            _frame.Navigated -= Frame_Navigated;
            _frame.Navigating -= Frame_Navigating;

            _frame.Navigated += Frame_Navigated;
            _frame.Navigating += Frame_Navigating;
        }

        /// <summary>
        /// Navigation Type
        /// </summary>
        /// <param name="navigationType"></param>
        /// <param name="navigationParameter"></param>
        /// <returns></returns>
        public bool Navigation(Type navigationType, object navigationParameter)
        {
            var returnValue = false;
            if (_frame == null) return false;

            returnValue = _frame.Navigate(navigationType, navigationParameter);

            return returnValue;
        }

        /// <summary>
        /// GoBack
        /// </summary>
        /// <returns></returns>
        public bool GoBack()
        {
            if (_frame.CanGoBack == true)
            {
                _frame.GoBack();
            }

            return true;
        }

        /// <summary>
        /// Navigation From
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Frame_Navigating(object sender, NavigatingCancelEventArgs e)
        {
            if(NavigatingCommand != null)
            {
                NavigatingCommand.Execute(e);
            }
            
        }

        /// <summary>
        /// Navigation To
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Frame_Navigated(object sender, NavigationEventArgs e)
        {
            if (NavigatedCommand != null)
            {
                NavigatedCommand.Execute(e);
            }
        }

    }
}
