﻿namespace UniversalSample.Commons
{
    public class ViewModelBase : BindableBase
    {
        private bool _isBusy;
        /// <summary>
        /// 작업중
        /// </summary>
        public bool IsBusy
        {
            get { return _isBusy; }
            set { _isBusy = value; OnPropertyChanged(); }
        }

        private string _title;
        /// <summary>
        /// 제목
        /// </summary>
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        /// <summary>
        /// 뷰가 로드시에 실행되는 메소드
        /// </summary>
        protected virtual void Loaded(object args) { }

        /// <summary>
        /// 뷰가 언로드시에 실행되는 메소드
        /// </summary>
        protected virtual void Unloaded(object args) { }

        /// <summary>
        /// Navigating시에 실행되는 메소드
        /// </summary>
        /// <param name="args"></param>
        protected virtual void Navigating(object args) { }

        /// <summary>
        /// Navigated시에 실행되는 메소드
        /// </summary>
        /// <param name="arg"></param>
        protected virtual void Navigated(object arg) { }

        /// <summary>
        /// 사이즈 변경 이벤트 처리 메소드
        /// </summary>
        protected virtual void SizeChanged(object args) { }

        private RelayCommand _loadedCommand;
        /// <summary>
        /// 로디드 커맨드
        /// </summary>
        public RelayCommand LoadedCommand
        {
            get { return _loadedCommand = _loadedCommand ?? new RelayCommand(Loaded); }
        }

        private RelayCommand _unloadedCommand;
        /// <summary>
        /// 언로드 커맨드
        /// </summary>
        public RelayCommand UnloadedCommand
        {
            get { return _unloadedCommand = _unloadedCommand ?? new RelayCommand(Unloaded); }
        }

        private RelayCommand _navigatingCommand;
        /// <summary>
        /// NavigatingCommand
        /// </summary>
        public RelayCommand NavigatingCommand
        {
            get { return _navigatingCommand = _navigatingCommand ?? new RelayCommand(Navigating); }
        }

        private RelayCommand _navigatedCommand;
        /// <summary>
        /// NavigatedCommand
        /// </summary>
        public RelayCommand NavigatedCommand
        {
            get { return _navigatedCommand = _navigatedCommand ?? new RelayCommand(Navigated); }
        }

        private RelayCommand _sizeChangedCommand;
        /// <summary>
        /// 사이즈 체인지 커맨드
        /// </summary>
        public RelayCommand SizeChangedCommand
        {
            get { return _sizeChangedCommand = _sizeChangedCommand ?? new RelayCommand(SizeChanged); }
        }

        private RelayCommand _goBackCommand;
        /// <summary>
        /// 백 커맨드
        /// </summary>
        public RelayCommand GoBackCommand
        {
            get { return _goBackCommand = _goBackCommand ?? new RelayCommand(args => FrameHelper.Instance.GoBack()); }
        }
    }
}
